"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@loopback/core");
const openapi_to_graphql_1 = require("openapi-to-graphql");
const graphql_1 = require("graphql");
let GraphQLService = class GraphQLService {
    constructor(ops) {
        this.ops = ops;
    }
    getAST(schema) {
        return graphql_1.parse(graphql_1.printSchema(schema));
    }
    reMapOperations(schema, ops) {
        const queries = [];
        const result = graphql_1.visit(this.getAST(schema), {
            enter: (node) => {
                if (node.name && node.name.value === 'Mutation') {
                    node.fields = node.fields.filter((field) => {
                        for (const op of ops) {
                            if (field.name.value === op.name) {
                                queries.push(field);
                                return false;
                            }
                        }
                        return true;
                    });
                    return node;
                }
                else if (node.name && node.name.value === 'Query') {
                    node.fields = [...node.fields, ...queries];
                    return node;
                }
            },
        });
        return graphql_1.buildASTSchema(result);
    }
    createOperationMapping(openAPISchemas, opsMap = []) {
        const ret = {};
        for (const schema of openAPISchemas) {
            const pathKeys = Object.keys(schema.paths);
            for (const path of pathKeys) {
                const op = opsMap.find(cuRop => {
                    return (cuRop.path === path &&
                        Object.keys(schema.paths[path]).includes(cuRop.method));
                });
                if (op) {
                    ret[schema.info.title] = Object.assign(Object.assign({}, ret[schema.info.title]), {
                        [op.path]: {
                            [op.method]: op.type === 'query' ? 0 : 1,
                        },
                    });
                }
            }
        }
        return ret;
    }
    async createSchema(cliArgs, openAPISchemas, opsMap = []) {
        const gqlOptions = {
            baseUrl: cliArgs.baseUrl || null,
            headers: cliArgs.headerObj || {},
            qs: cliArgs.queryObj || {},
            fillEmptyResponses: cliArgs.fill,
            addLimitArgument: cliArgs.limit,
            strict: cliArgs.strict,
            operationIdFieldNames: cliArgs.strict,
            provideErrorExtensions: cliArgs.errors,
            equivalentToMessages: cliArgs.errors,
            selectQueryOrMutationField: this.createOperationMapping(openAPISchemas, opsMap),
        };
        const { schema, report } = await openapi_to_graphql_1.createGraphQlSchema(openAPISchemas, gqlOptions);
        return Promise.resolve({ schema, report });
    }
};
GraphQLService = __decorate([
    core_1.bind({ scope: core_1.BindingScope.TRANSIENT }),
    __param(0, core_1.inject('ops')),
    __metadata("design:paramtypes", [Object])
], GraphQLService);
exports.GraphQLService = GraphQLService;
//# sourceMappingURL=graphql.service.js.map