"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@loopback/core");
const oasValidator = require('openapi-enforcer');
const util_1 = require("util");
const YAML = require("yaml");
const fs = require("fs");
const request_promise_native_1 = require("request-promise-native");
const joi_1 = require("@hapi/joi");
let OpenApiService = class OpenApiService {
    constructor() { }
    addParameters(oasData, paramsToAdd) {
        for (const oas of oasData) {
            const paths = Object.keys(oas.paths);
            for (const path of paths) {
                const methods = Object.keys(oas.paths[path]);
                for (const method of methods) {
                    oas.paths[path][method].parameters =
                        oas.paths[path][method].parameters || [];
                    const ownParams = oas.paths[path][method].parameters;
                    for (const paramStr of paramsToAdd) {
                        const param = paramStr.split(':');
                        if (!ownParams.find((p) => p.name === param[2]) &&
                            !ownParams.find((p) => p.$ref)) {
                            ownParams.push({
                                in: param[0],
                                schema: { type: param[1] },
                                name: param[2],
                            });
                        }
                    }
                }
            }
        }
    }
    async validate(paths) {
        for (const path of paths) {
            oasValidator(path, { fullResult: true }).then((error, warning) => {
                if (!error) {
                    if (warning) {
                        throw new Error(`OAS produced warnings: ${warning}`);
                    }
                }
                else {
                    throw new Error(`OAS is invalid: ${util_1.inspect(error, {
                        showHidden: false,
                        depth: null,
                    })}`);
                }
            });
        }
        return Promise.resolve(true);
    }
    async load(paths) {
        const oasData = [];
        const uriSchema = joi_1.string().uri({ scheme: ['http', 'https'] });
        try {
            for (const path of paths) {
                const isURL = Boolean(!uriSchema.validate(path).error);
                if (!isURL) {
                    oasData.push(fs.readFileSync(path, { encoding: 'utf8' }).toString());
                }
                else {
                    oasData.push(await request_promise_native_1.get(path));
                }
            }
        }
        catch (e) {
            throw new Error(`Could not load file or URL: "${e}".`);
        }
        return Promise.resolve(oasData);
    }
    parse(oasData) {
        return Promise.resolve(oasData.map((oas) => {
            try {
                return JSON.parse(oas);
            }
            catch (e) {
                try {
                    return YAML.parse(oas);
                }
                catch (error) {
                    throw new Error(`Could not parsed OAS, should YAML or JSON: "${error}"`);
                }
            }
        }));
    }
};
OpenApiService = __decorate([
    core_1.bind({ scope: core_1.BindingScope.TRANSIENT }),
    __metadata("design:paramtypes", [])
], OpenApiService);
exports.OpenApiService = OpenApiService;
//# sourceMappingURL=open-api.service.js.map